C***********************************************************************
C***********************************************************************
C**
C** INTEGRAL SYSTEMS, INC.
C**
C***********************************************************************
C**
C** PROJECT : OPERATIONS GROUND EQUIPMENT FOR GOES-NEXT
C** SYSTEM : EARTH LOCATION USERS GUIDE
C** ROUTINE : LMODEL
C** SOURCE : LMODEL.FOR
C** LOAD NAME : ANY
C** PROGRAMMER: IGOR LEVINE
C**
C** VER. DATA BY COMMENT
C** ---- -------- --- ---------------------------------------------
C** 1 01/09/89 IL INITIAL CREATION
C** 2 06/02/89 IL COORDINATE AXES CHANGED ACCORDING TO
C** FORD'S DEFINITION IN SDAIP, DRL 504-01
C** 3 08/21/89 IL CORRECTED ORBIT ANGLE COMPUTATIONS
C** 4 02/27/90 IL S/C COMPENSATION APPLIED UNCONDITIONALLY;
C** REFERENCE RADIAL DISTANCE, LATITUDE AND
C** ORBIT YAW SET TO ZERO IF IMC DISABLED.
C** 5 12/01/93 IL ADDED TRAP FOR SLAT=SYAW=0; CORRECTED
C** EXPRESSION FOR LAM.
C**
C***********************************************************************
C**
C** THIS SUBROUTINE COMPUTES THE POSITION OF THE SATELLITE AND THE
C** ATTITUDE OF THE IMAGER OR SOUNDER. THE CALCULATIONS ARE BASED
C** ON THE OATS ORBIT AND ATTITUDE MODEL REPRESENTED BY THE O&A
C** PARAMETER SET IN GVAR BLOCK 0.
C** INPUTS:
C** TIME, EPOCH TIME, O&A PARAMETER SET, IMC STATUS.
C**
C** OUTPUTS:
C** THE SPACECRAFT POSITION VECTOR IN EARTH-FIXED COORDINATES;
C** THE GEOMETRIC ROLL, PITCH, YAW ANGLES AND THE ROLL,
C** PITCH MISALIGNMENTS FOR EITHER THE IMAGER OR THE SOUNDER;
C** THE EARTH-FIXED TO INSTRUMENT FRAME TRANSFORMATION MATRIX;
C** GEOGRAPHIC LATITUDE AND LONGITUDE AT SUBSATELLITE POINT.
C**
C** DESCRIPTION
C** LMODEL ACCEPTS AN INPUT DOUBLE PRECISION TIME IN MINUTES FROM
C** 1950, JAN.1.0 AND AN INPUT SET OF O&A PARAMETERS AND COMPUTES
C** POSITION OF THE SATELLITE, THE ATTITUDE ANGLES AND ATTITUDE
C** MISALIGNMENTS AND THE INSTRUMENT TO EARTH-FIXED COORDINATES
C** TRANSFORMATION MATRIX.
C**
C***********************************************************************
C**
C** CALLED BY : ANY
C** COMMONS MODIFIED: /ELCOMM/ XS,Q3,PITCH,ROLL,YAW,PMA,RMA,BT
C** INPUTS : NONE
C** OUTPUTS : NONE
C** ROUTINES CALLED : INST2ER,GATT
C**
C***********************************************************************
C***********************************************************************
      SUBROUTINE LMODEL(T,TU,REC,IMC,RLAT,RLON)
      IMPLICIT NONE
C
C CALLING ARGUMENTS
C
      REAL*8 T
C INPUT TIME FROM 1950, JAN 1.0 (MIN)
      REAL*8 TU
C EPOCH TIME FROM 1950, JAN 1.0 (MIN)
      REAL*4 REC(336)
C INPUT O&A PARAMETER SET
      INTEGER IMC
C INPUT IMC STATUS: 0 - ON, 1 - OFF
      REAL*4 RLAT
C SUBSATELLITE GEODETIC LATITUDE (RAD)
      REAL*4 RLON
C SUBSATELLITE LONGITUDE IN RADIANS
C
C LOCAL VARIABLES
C
      REAL*8 R
C NORMALIZED SATELLITE DISTANCE (IN UNITS OF KMER9)
      REAL*8 TS
C TIME FROM EPOCH IN MINUTES
      REAL*8 B(3,3)
C SPACCRAFT TO EARTH-FIXED COORDINATES TRANSFORMATION
C MATRIX
      REAL*4 TE
C EXPONENENTIAL TIME DELAY FROM EPOCH (IN MINUTES)
      REAL*4 PHI
C SUBSATELLITE GEOCENTRIC LATITUDE IN RADIANS
      REAL*8 DR
C RADIAL DISTANCE FROM THE NOMINAL (KM)
      REAL*4 PSI
C ORBITAL YAW (IN RADIANS)
      REAL*8 LAM
C IMC LONGITUDE (IN RADIANS)
      REAL*4 U
C ARGUMENT OF LATITUDE (IN RADIANS)
      REAL*4 SU,CU
C SIN(U), COS(U)
      REAL*4 SI,CI
C SINE AND COSINE OF THE ORBIT INCLINATION
      REAL*4 SLAT
C SINE OF GEOCENTRIC LATITUDE
      REAL*4 ASC
C LONGITUDE OF THE ASCENDING NODE IN RADIANS
      REAL*4 SA,CA
C SINE AND COSINE OF ASC
      REAL*4 SYAW
C SINE OF THE ORBIT YAW
      REAL*4 WA
C SOLAR ORBIT ANGLE IN RADIANS
      REAL*4 W
C ORBIT ANGLE IN RADIANS
      REAL*4 SW,CW
C SIN(W), COS(W)
      REAL*4 S2W,C2W
C SIN(2*W), COS(2*W)
      REAL*4 SW1,CW1
C SIN(0.927*W), COS(0.927*W)
      REAL*4 SW3,CW3
C SINE AND COSINE OF 1.9268*W
      REAL*8 DLAT
C CHANGE IN SINE OF GEOCENTRIC LATITUDE
      REAL*8 DYAW
C CHANGE IN SINE OF ORBIT YAW
      REAL*4 GATT
C SUBROUTINE FUNCTION
      REAL*4 A1,A2
C WORK AREAS
C
C INCLUDE FILES
C
      INCLUDE 'ELCONS.INC'
      INCLUDE 'ELCOMM.INC'
C
C***********************************************************************
C
C ASSIGN REFERENCE VALUES TO THE SUBSATELLITE LONGITUDE AND
C LATITUDE, THE RADIAL DISTANCE AND THE ORBIT YAW.
C
      write (*,*) T,TU,IMC,RLAT,RLON
      LAM=REC(5)
      DR=REC(6)
      PHI=REC(7)
      PSI=REC(8)
C
C ASSIGN REFERENCE VALUES TO THE ATTITUDES AND MISALIGNMENTS
C
      ROLL=REC(9)
      PITCH=REC(10)
      YAW=REC(11)
      RMA=0.
      PMA=0.
C
C IF IMC IS OFF, COMPUTE CHANGES IN THE SATELLITE ORBIT
C
      IF (IMC.NE.0) THEN
C
C SET REFERENCE RADIAL DISTANCE, LATITUDE AND ORBIT YAW TO ZERO
C
      DR=0.
      PHI=0.
      PSI=0.
C
C COMPUTE TIME SINCE EPOCH (IN MINUTES)
C
      TS=T-TU
C
C COMPUTES ORBIT ANGLE AND THE RELATED TRIGONOMETRIC FUNCTIONS.
C EARTH ROTATIONAL RATE=.7292115E-4 (rad/s)
C
      W=0.7292115D-4*60.0D0*TS
      SW=SIN(W)
      CW=COS(W)
      SW1=SIN(0.927*W)
      CW1=COS(0.927*W)
      S2W=SIN(2.*W)
      C2W=COS(2.*W)
      SW3=SIN(1.9268*W)
      CW3=COS(1.9268*W)
C
C COMPUTES CHANGE IN THE IMC LONGITUDE FROM THE REFERENCE
C
      LAM=LAM+REC(18)+(REC(19)+REC(20)*W)*W
     1 +(REC(27)*SW1+REC(28)*CW1+REC(21)*SW+REC(22)*CW
     2 +REC(23)*S2W+REC(24)*C2W + REC(25)*SW3+REC(26)*CW3
     3 +W*(REC(29)*SW+REC(30)*CW))*2.
C
C COMPUTES CHANGE IN RADIAL DISTANCE FROM THE REFERENCE (KM)
C
      DR=DR + REC(31) + REC(32)*CW+REC(33)*SW
     1 +REC(34)*C2W+REC(35)*S2W + REC(36)*CW3+REC(37)*SW3
     2 +REC(38)*CW1+REC(39)*SW1 + W*(REC(40)*CW+REC(41)*SW)
C
C COMPUTES THE SINE OF THE CHANGE IN THE GEOCENTRIC LATITUDE
C
      DLAT=REC(42) + REC(43)*CW+REC(44)*SW
     1 +REC(45)*C2W+REC(46)*S2W
     2 +W*(REC(47)*CW+REC(48)*SW)
     3 +REC(49)*CW1+REC(50)*SW1
C
C COMPUTES GEOCENTRIC LATITUDE BY USING AN EXPANSION FOR ARCSINE
C
      PHI=PHI+DLAT*(1.+DLAT*DLAT/6.)
C
C COMPUTES SINE OF THE CHANGE IN THE ORBIT YAW
C
      DYAW=REC(51) + REC(52)*SW+REC(53)*CW
     1 +REC(54)*S2W+REC(55)*C2W
     2 +W*(REC(56)*SW+REC(57)*CW)
     3 +REC(58)*SW1+REC(59)*CW1
C
C COMPUTES THE ORBIT YAW BY USING AN EXPANSION FOR ARCSINE.
C
      PSI=PSI+DYAW*(1.+DYAW*DYAW/6.)
C
C CALCULATION OF CHANGES IN THE SATELLITE ORBIT ENDS HERE
C
      END IF
C
C CONVERSION OF THE IMC LONGITUDE AND ORBIT YAW TO THE SUBSATELLITE
C LONGITUDE AND THE ORBIT INCLINATION (REF: GOES-PCC-TM-2473, INPUTS
C REQUIRED FOR EARTH LOCATION AND GRIDDING BY SPS, JUNE 6, 1988)
C
      SLAT=SIN(PHI)
      SYAW=SIN(PSI)
      SI=SLAT**2+SYAW**2
      CI=SQRT(1.-SI)
      SI=SQRT(SI)
      IF (SLAT.EQ.0. .AND. SYAW.EQ.0.) THEN
            U=0.
      ELSE
            U=ATAN2(SLAT,SYAW)
      END IF
      SU=SIN(U)
      CU=COS(U)
C
C COMPUTES LONGITUDE OF THE ASCENDING NODE
C
      ASC=LAM-U
      SA=SIN(ASC)
      CA=COS(ASC)
C
C COMPUTES THE SUBSATELLITE GEOGRAPHIC LATITUDE
C
      RLAT=ATAN(AEBE2*TAN(PHI))
C
C COMPUTES THE SUBSATELLITE LONGITUDE
C
      RLON=ASC+ATAN2(CI*SU,CU)
C
C COMPUTES THE SPACECRAFT TO EARTH-FIXED COORDINATES TRANSFORMATION
C MATRIX:
C (VECTOR IN ECEF COORDINATES) = B * (VECTOR IN S/C COORDINATES)
C
      B(1,2)=-SA*SI
      B(2,2)= CA*SI
      B(3,2)=-CI
      B(1,3)=-CA*CU+SA*SU*CI
      B(2,3)=-SA*CU-CA*SU*CI
      B(3,3)=-SLAT
      B(1,1)=-CA*SU-SA*CU*CI
      B(2,1)=-SA*SU+CA*CU*CI
      B(3,1)= CU*SI
C
C COMPUTES THE NORMALIZED SPACECRAFT POSITION VECTOR IN EARTH-FIXED
C COORDINATES - XS.
C
      R=(NOMORB+DR)/AE
      XS(1)=-B(1,3)*R
      XS(2)=-B(2,3)*R
      XS(3)=-B(3,3)*R
C
C PRECOMPUTES Q3 (USED IN LPOINT)
C
      Q3=XS(1)**2+XS(2)**2+AEBE2*XS(3)**2-1.0
C
C COMPUTES THE ATTITUDES AND MISALIGNMENTS IF IMC IS OFF
C
      IF (IMC.NE.0) THEN
C
C COMPUTES THE SOLAR ORBIT ANGLE
C
      WA=REC(60)*TS
C
C COMPUTES THE DIFFERENCE BETWEEN CURRENT TIME, TS, AND THE
C EXPONENTIAL TIME, REC(61). NOTE THAT BOTH TIMES ARE SINCE EPOCH.
C
      TE=TS-REC(61)
C
C COMPUTES ROLL
C
      ROLL=ROLL+GATT(62,REC,WA,TE)
C
C COMPUTES PITCH
C
      PITCH=PITCH+GATT(117,REC,WA,TE)
C
C COMPUTES YAW
C
      YAW=YAW+GATT(172,REC,WA,TE)
C
C COMPUTES ROLL MISALIGNMENT
C
      RMA=GATT(227,REC,WA,TE)
C
C COMPUTES PITCH MISALIGNMENT
C
      PMA=GATT(282,REC,WA,TE)
C
C APPLY THE SPACECRAFT COMPENSATION
C
      ROLL=ROLL+REC(15)
      PITCH=PITCH+REC(16)
      YAW=YAW+REC(17)
      END IF
C
C COMPUTES THE INSTRUMENT TO EARTH-FIXED COORDINATES TRANSFORMATION
C MATRIX - BT
C
      CALL INST2ER(ROLL,PITCH,YAW,B,BT)
      RETURN
      END
