C***********************************************************************
C***********************************************************************
C**
C** INTEGRAL SYSTEMS, INC.
C**
C***********************************************************************
C**
C** PROJECT : OPERATIONS GROUND EQUIPMENT FOR GOES-NEXT
C** SYSTEM : EARTH LOCATION USERS GUIDE
C** ROUTINE : LPOINT
C** SOURCE : LPOINT.FOR
C** LOAD NAME : ANY
C** PROGRAMMER: IGOR LEVINE
C**
C** VER. DATA BY COMMENT
C** ---- -------- --- ---------------------------------------------
C** 1 01/09/89 IL INITIAL CREATION
C** 2 06/02/89 IL COORDINATE AXES CHANGED ACCORDING TO
C** FORD'S DEFINITION IN SDAIP, DRL504-01
C** 3 12/01/93 IL IMPLEMENTED NEW FORMULAE FOR SCAN ANGLE
C** CORRECTIONS DUE TO MISALIGNMENTS
C**
C***********************************************************************
C**
C** THIS SUBROUTINE CONVERTS THE INSTRUMENT ELEVATION AND SCAN
C** ANGLES TO THE RELATED GEOGRAPHIC LATITUDE AND LONGITUDE.
C**
C***********************************************************************
C**
C** CALLED BY : ANY
C** COMMONS MODIFIED: NONE
C** INPUTS : NONE
C** OUTPUTS : NONE
C** ROUTINES CALLED : NONE
C**
C***********************************************************************
C***********************************************************************
      SUBROUTINE LPOINT(INSTR,FLIP_FLG,ALPHA0,ZETA0,RLAT,RLON,IERR)
      IMPLICIT NONE
C
C CALLING PARAMETERS
C
      INTEGER*4 INSTR
C INSTRUMENT CODE (1=IMAGER,2=SOUNDER)
      INTEGER*4 FLIP_FLG
C S/C ORIENTATION FLAG (1=NORMAL,-1=INVERTED)
      REAL*4 ALPHA0
c ELEVATION ANGLE (RAD)
      REAL*4 ZETA0
c SCAN ANGLE (RAD)
      REAL*4 RLAT
c LATITUDE IN RADIANS (OUTPUT)
      REAL*4 RLON
c LONGITUDE IN RADIANS (OUTPUT)
      INTEGER IERR
c OUTPUT STATUS; 0 - POINT ON THE EARTH
c 1 - INSTRUMENT POINTS OFF EARTH
C
C LOCAL VARIABLES
C
      REAL*8 G1(3)
c POINTING VECTOR IN EARTH-CENTERED COORDINATES
      REAL*8 H
c SLANT DISTANCE TO THE EARTH POINT (KM)
      REAL*8 Q1,Q2,D
c WORK SPACE
      REAL*4 G(3)
c POINTING VECTOR IN INSTRUMENT COORDINATES
      REAL*4 U(3)
c COORDINATES OF THE EARTH POINT (KM)
      REAL*4 SA,CA,DA,DZ,D1,CZ,SZ,FF,DOFF,ALPHA,ZETA
c WORK SPACE
C
C INCLUDE FILES
C
      INCLUDE 'ELCONS.INC'
      INCLUDE 'ELCOMM.INC'
      INCLUDE 'INSTCOMM.INC'
C***********************************************************************
      IERR=1
C
C COMPUTE SIGN OF MISALIGNMENT CORRECTIONS AND ORIGIN OFFSET CORRECTIONS
C
      FF = FLIP_FLG
      IF (INSTR.EQ.2) FF = - FF
            DOFF = SCNMAX(INSTR) - EWNOM(INSTR)
C
C ADD THE NEW SECOND ORDER ORIGIN OFFSET CORRECTION
C
      ALPHA = ALPHA0 - ALPHA0*ZETA0*DOFF
      ZETA = ZETA0 + 0.5*ALPHA0*ALPHA0*DOFF
C
C COMPUTES TRIGONOMETRIC FUNCTIONS OF THE SCAN AND ELEVATION
C ANGLES CORRECTED FOR THE ROLL AND PITCH MISALIGNMENTS
C
      CA=COS(ALPHA)
      SA=SIN(ALPHA)
      CZ=COS(ZETA)
      DA=ALPHA-PMA*SA*(FF/CZ+TAN(ZETA))-RMA*(1.-CA/CZ)
      DZ=ZETA+FF*RMA*SA
C
C COMPUTES POINTING VECTOR IN INSTRUMENT COORDINATES
C
      CZ=COS(DZ)
      G(1)=SIN(DZ)
      G(2)=-CZ*SIN(DA)
      G(3)=CZ*COS(DA)
C
C TRANSFORMS THE POINTING VECTOR TO EARTH-FIXED COORDINATES
C
      G1(1)=BT(1,1)*G(1)+BT(1,2)*G(2)+BT(1,3)*G(3)
      G1(2)=BT(2,1)*G(1)+BT(2,2)*G(2)+BT(2,3)*G(3)
      G1(3)=BT(3,1)*G(1)+BT(3,2)*G(2)+BT(3,3)*G(3)
C
C COMPUTES COEFFICIENTS AND SOLVES A QUADRATIC EQUATION TO
C FIND THE INTERSECT OF THE POINTING VECTOR WITH THE EARTH
C SURFACE
C
      Q1=G1(1)**2+G1(2)**2+AEBE2*G1(3)**2
      Q2=XS(1)*G1(1)+XS(2)*G1(2)+AEBE2*XS(3)*G1(3)
      D=Q2*Q2-Q1*Q3
      IF (DABS(D).LT.1.D-9) D=0.
C
C IF THE DISCIMINANTE OF THE EQUATION, D, IS NEGATIVE, THE
C INSTRUMENT POINTS OFF THE EARTH
C
      IF (D.LT.0) THEN
            RLAT=999999.
            RLON=999999.
      RETURN
      END IF
      D=DSQRT(D)
C
C SLANT DISTANCE FROM THE SATELLITE TO THE EARTH POINT
C
      H=-(Q2+D)/Q1
C
C CARTESIAN COORDINATES OF THE EARTH POINT
C
      U(1)=XS(1)+H*G1(1)
      U(2)=XS(2)+H*G1(2)
      U(3)=XS(3)+H*G1(3)
C
C SINUS OF GEOCENTRIC LATITUDE
C
      D1=U(3)/SQRT(U(1)**2+U(2)**2+U(3)**2)
C
C GEOGRAPHIC (GEODETIC) COORDINATES OF THE POINT
C
      RLAT=ATAN(AEBE2*D1/SQRT(1.-D1*D1))
      RLON=ATAN2(U(2),U(1))
      IERR=0
      RETURN
      END
